/////////////////////////////////////////////////////////////////////////////
//                                                            (c) 1998/9 SNS
//  Configure
//                  Supporting code for TreeShare/PilotShare
//
//      Authors: George A. Smith & John R. Coleman
//      Why:     We felt the need.
//
//  LICENCE CONDITIONS:
//
//      The Authors consent to GNU General Public Licence for this code.
//      Other Licencing arrangements are possible.
//
//  NOTES:
//
//      Please notify us of any problems you find in the code.
//      Please notify us of any changes you make to this code that you
//              subsequently redistribute.
//      If you use this program, please tell us...
//
//      We can be contacted at:     PilotShare@SNS.to
//                          or:     TreeShare@SNS.to
//
//      Info:       http://SNS.to/PilotShare
//
//      If you redistribute, please leave the above in!
//
/////////////////////////////////////////////////////////////////////////////

#include "IncCmn.h"
//---------------------------------------------------------------------------
#include "Configure.h"
//---------------------------------------------------------------------------

#include "PersistentData.h"
#include "WinTools.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)
#pragma resource "*.dfm"

static void selectFolder( TEdit * te, char * title, HWND parent )
{
    char dp[ MAX_PATH+2 ];

    te->GetTextBuf( dp, MAX_PATH );
    te->SetTextBuf( browseForFolder( dp, title, parent ) );
}

TTreeShare *TreeShare;

//---------------------------------------------------------------------------

__fastcall TTreeShare::TTreeShare( TLogo * Owner) : TForm(Owner)
{
    ourOwner = Owner;
    textboxHold = NULL;
}

//---------------------------------------------------------------------------

void __fastcall TTreeShare::bt_BrowseRunCommandClick(TObject *Sender)
{
    if ( FindRunProg->Execute() )
        tb_RunCommand->SetTextBuf( FindRunProg->FileName.c_str() );
}

//---------------------------------------------------------------------------

void __fastcall TTreeShare::bt_BrowweLockFolderClick(TObject *Sender)
{
    selectFolder( tb_LockFolder, "Select a Remote Lock Folder", Handle );
}

//---------------------------------------------------------------------------

void __fastcall TTreeShare::tb_BrowseLocalFolderClick(TObject *Sender)
{
    selectFolder( tb_LocalFolder, "Select a Local Data Folder", Handle );

}

//---------------------------------------------------------------------------

void __fastcall TTreeShare::bt_BrowseSharedFolderClick(TObject *Sender)
{
    selectFolder( tb_SharedFolder, "Select a Remote Shared Data Folder", Handle );
}

//---------------------------------------------------------------------------

#pragma argsused
void __fastcall TTreeShare::keyProcess( char key, TEdit * srcEdit, char *origText, TWinControl * changedTo )
{
    if ( 13 == key )    {   changedTo->SetFocus();   return; }
    if ( 27 == key )
    {
        if ( 0 != strcmp( getBoxText(srcEdit),  origText ) )
            srcEdit->SetTextBuf( origText );
        else
            Close();

        return;
    }

}

//---------------------------------------------------------------------------

void __fastcall TTreeShare::tb_LockFolderKeyPress(TObject *Sender, char &Key)
{
    keyProcess( Key, tb_LockFolder, textboxHold, tb_SharedFolder );
}

//---------------------------------------------------------------------------

void __fastcall TTreeShare::tb_SharedFolderKeyPress(TObject *Sender, char &Key)
{
    keyProcess( Key, tb_SharedFolder, textboxHold, tb_LocalFolder );
}

//---------------------------------------------------------------------------

void __fastcall TTreeShare::tb_LocalFolderKeyPress(TObject *Sender, char &Key)
{
    keyProcess( Key, tb_LocalFolder, textboxHold, tb_RunCommand );
}

//---------------------------------------------------------------------------

void __fastcall TTreeShare::tb_RunCommandKeyPress(TObject *Sender, char &Key)
{
    keyProcess( Key, tb_RunCommand, textboxHold, bt_Save );
}

//---------------------------------------------------------------------------

static bool hasFiles( char * path )
{
    char * tp = path + strlen( path ) - 1;
    if ( '\\' == *tp || '/' == *tp )    *tp = 0;

    path = dynSpace( path, path, "\\*.*" );

    bool rv = ( NULL != findFile( path ));
    dynFree( path );
    return rv;
}

void __fastcall TTreeShare::bt_SaveClick(TObject *Sender)
{
    if ( ! ValidateFields() )     return;

    dynFree( setRemoteLock( getBoxText( tb_LockFolder ) ));
    dynFree( setRemoteSharedData( getBoxText( tb_SharedFolder ) ));
    dynFree( setLocalData( getBoxText( tb_LocalFolder ) ));
    dynFree( setRunCommand( getBoxText( tb_RunCommand ) ));
    dynFree( setRunCommandOption( getBoxText( tb_RunCommandOptions ) ));

    char * tp = writeData();

    if ( NULL != tp )
    {
        lb_error->SetTextBuf( tp );
        return;
    }

    if ( hasFiles( getBoxText( tb_LocalFolder ) ) )
        if ( ! hasFiles( getBoxText( tb_SharedFolder ) ) )
            if ( IDYES == MessageBox( Handle, "Copy files from Local to Remote Shared Data folder",
                                            "Remote Shared folder empty",
                                            MB_YESNO|MB_ICONEXCLAMATION|MB_APPLMODAL  )
               )
                if ( ! folderCopyWithOldSave( getLocalData(), getRemoteSharedData() ) )
                {
                    lb_error->SetTextBuf( "Error copying local data to Remote Shared Data Folder" );
                    return;
                }
    Close();
}

//---------------------------------------------------------------------------

void __fastcall TTreeShare::FormCreate(TObject *Sender)
{
    char * run = getRunCommand();
    char * pilot;

    if ( 0 == strlen( run ) )
        if ( 0 == strcmpi( ourOwner->ourName, "PILOTSHARE" ) )
            if ( validateCommand( pilot = "c:\\Palm\\PALM.exe" ) )
                run = pilot;
    tb_RunCommand->SetTextBuf( run );

    tb_RunCommandOptions->SetTextBuf( getRunCommandOption() );
    tb_LocalFolder->SetTextBuf( getLocalData() );
    tb_SharedFolder->SetTextBuf( getRemoteSharedData() );
    tb_LockFolder->SetTextBuf( getRemoteLock() );
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

bool __fastcall TTreeShare::ValidateFields( void )
{
    char * lck     = getBoxText( tb_LockFolder );
    char * shared  = getBoxText( tb_SharedFolder );
    char * local   = getBoxText( tb_LocalFolder );
    char * run     = getBoxText( tb_RunCommand );

    int ret = dataValidate( lck, shared, local, run );

    if ( 0 == ret )
        if ( 0 == strcmpi( ourOwner->ourName, "PILOTSHARE" ) )
            if ( 0 == strcmpi( getBaseName( run ), "PILOT" ) )
                ret = 100;

    delete [] lck;
    delete [] shared;
    delete [] local;
    delete [] run;

    char * text;
    switch( ret )
    {
        case 0:
        default:    return true;

        case 1:     tb_LockFolder->SetFocus();
                    text = "Remote Lock Folder NOT found!";
                    break;
        case 2:     tb_SharedFolder->SetFocus();
                    text = "Remote Shared Data Folder NOT found!";
                    break;
        case 3:     tb_LocalFolder->SetFocus();
                    text = "Local Data Folder NOT found!";
                    break;
        case 4:     tb_RunCommand->SetFocus();
                    text = "Run Command NOT found!";
                    break;
        case 100:   tb_RunCommand->SetFocus();
                    text = "PILOT.exe does not work with PilotShare!";
                    break;
    }
    lb_error->SetTextBuf( text );
    return false;
}

void __fastcall TTreeShare::FormShow(TObject *Sender)
{
    ValidateFields();
}

//---------------------------------------------------------------------------

void __fastcall TTreeShare::bt_CancelClick(TObject *Sender)
{
    ourOwner->setCancel();
    Close();
}

//---------------------------------------------------------------------------

char * __fastcall TTreeShare::getBoxText( TControl * box )
{
    char * tp;
    int i;
    tp = new char [ i = box->GetTextLen()+1 ];
    box->GetTextBuf( tp ,i );

    return tp;
}

void __fastcall TTreeShare::tb_LockFolderEnter(TObject *Sender)
{
    if ( NULL != textboxHold )  delete [] textboxHold;
    textboxHold = getBoxText( tb_LockFolder );
}

//---------------------------------------------------------------------------

void __fastcall TTreeShare::tb_SharedFolderEnter(TObject *Sender)
{
    if ( NULL != textboxHold )  delete [] textboxHold;
    textboxHold = getBoxText( tb_SharedFolder );
}

//---------------------------------------------------------------------------

void __fastcall TTreeShare::tb_LocalFolderEnter(TObject *Sender)
{
    if ( NULL != textboxHold )  delete [] textboxHold;
    textboxHold = getBoxText( tb_LocalFolder );
}

//---------------------------------------------------------------------------

void __fastcall TTreeShare::tb_RunCommandEnter(TObject *Sender)
{
    if ( NULL != textboxHold )  delete [] textboxHold;
    textboxHold = getBoxText( tb_RunCommand );
}

//---------------------------------------------------------------------------

void __fastcall TTreeShare::tb_RunCommandOptionsEnter(TObject *Sender)
{
    if ( NULL != textboxHold )  delete [] textboxHold;
    textboxHold = getBoxText( tb_RunCommandOptions );
}

//---------------------------------------------------------------------------
