/////////////////////////////////////////////////////////////////////////////
//                                                            (c) 1998/9 SNS
//  PersistentData
//                  Supporting code for TreeShare/PilotShare
//
//      Authors: George A. Smith & John R. Coleman
//      Why:     We felt the need.
//
//  LICENCE CONDITIONS:
//
//      The Authors consent to GNU General Public Licence for this code.
//      Other Licencing arrangements are possible.
//
//  NOTES:
//
//      Please notify us of any problems you find in the code.
//      Please notify us of any changes you make to this code that you
//              subsequently redistribute.
//      If you use this program, please tell us...
//
//      We can be contacted at:     PilotShare@SNS.to
//                          or:     TreeShare@SNS.to
//
//      Info:       http://SNS.to/PilotShare
//
//      If you redistribute, please leave the above in!
//
/////////////////////////////////////////////////////////////////////////////

#include "IncCmn.h"
//---------------------------------------------------------------------------
#include "PersistentData.h"
//---------------------------------------------------------------------------

#include <io.h>

#include "WinErr.h"
#include "WinTools.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

#define STOR_LEN          (MAX_PATH+2)

static char * remoteLock = NULL;
static char * remoteSharedData = NULL;
static char * localData = NULL;
static char * runCommand = NULL;
static char * runCommandOptions = NULL;

#define SECTION                 "RunTime"
#define REMOTE_LOCK_KEY         "RemoteLockFolder"
#define REMOTE_SHARED_DATA_KEY  "RemoteSharedDataFolder"
#define LOCAL_DATA_KEY          "LocalDataFolder"
#define RUN_COMMAND_KEY         "RunCommand"
#define RUN_COMMAND_OPTIONS_KEY "RunCommandOptions"

#define NO_NULL(a)  (NULL==a?"":removeTrailingSlash(a))

//----------------------------------------------------------------------------

char * removeTrailingSlash( char * buf )
{
    char * tp = buf + strlen( buf ) - 1;
    if ( ('\\' == *tp) && (*tp == '/') )
        *tp   = 0;

    return buf;
}

char * getRemoteLock( void )        {   return ( NO_NULL(remoteLock) ); }
char * getRemoteSharedData( void )  {   return ( NO_NULL(remoteSharedData) ); }
char * getLocalData( void )         {   return ( NO_NULL(localData) ); }
char * getRunCommand( void )        {   return ( NO_NULL(runCommand) ); }
char * getRunCommandOption( void )  {   return ( NO_NULL(runCommandOptions) ); }

char * setRemoteLock( char * data )
{
    if ( NULL != remoteLock )  delete [] remoteLock;
    remoteLock = newSpace( data );
    return data;
}

char * setRemoteSharedData( char * data )
{
    remoteSharedData = dynSpace( remoteSharedData, data );
    return data;
}

char * setLocalData( char * data )
{
    localData = dynSpace( localData, data );
    return data;
}

char * setRunCommand( char * data )
{
    runCommand = dynSpace( runCommand, data );
    return data;
}

char * setRunCommandOption( char * data )
{
    runCommandOptions = dynSpace( runCommandOptions, data );
    return data;
}

//----------------------------------------------------------------------------

static char * iniName = NULL;

char * buildIniName( char * exePath )
{
    iniName = dynFree( iniName );

    iniName = new char[ strlen( exePath ) + 5 ];
    strcpy( iniName, exePath );

    char *tp = iniName + strlen( iniName );
    while( *tp != '.' && tp > iniName )     tp--;

    if ( '.' != *tp )   tp = iniName + strlen( iniName );

    strcpy( tp, ".Ini" );

    return iniName;
}

// ----------------------------------------------------------------------------

static char * getIniName( void )
{
    extern char *_argv[];

    if ( NULL != iniName )
        if ( *iniName )     return iniName;

    return buildIniName( _argv[0] );
}

char * writeData( void )
        // returns NULL on no error
        //      otherwsie error message string
{
    static char errBuf[200];
    if ( 0 == WritePrivateProfileString( SECTION, REMOTE_LOCK_KEY,         remoteLock,       getIniName() ) )
    {
        strcpy( errBuf, "Error updating RemoteLockFolder - " );
        strcat( errBuf, DecodeWindowsError(GetLastError()) );
        return errBuf;
    }

    if ( 0 == WritePrivateProfileString( SECTION, REMOTE_SHARED_DATA_KEY,  remoteSharedData, getIniName() ) )
    {
        strcpy( errBuf, "Error updating RemoteShareFolder - " );
        strcat( errBuf, DecodeWindowsError(GetLastError()) );
        return errBuf;
    }

    if ( 0 == WritePrivateProfileString( SECTION, LOCAL_DATA_KEY,          localData,        getIniName() ) )
    {
        strcpy( errBuf, "Error updating LocalDataFolder - " );
        strcat( errBuf, DecodeWindowsError(GetLastError()) );
        return errBuf;
    }

    if ( 0 == WritePrivateProfileString( SECTION, RUN_COMMAND_KEY,         runCommand,       getIniName() ) )
    {
        strcpy( errBuf, "Error updating RunCommand - " );
        strcat( errBuf, DecodeWindowsError(GetLastError()) );
        return errBuf;
    }

    if ( 0 == WritePrivateProfileString( SECTION, RUN_COMMAND_OPTIONS_KEY, runCommandOptions,getIniName() ) )
    {
        strcpy( errBuf, "Error updating RunCommandOptions - " );
        strcat( errBuf, DecodeWindowsError(GetLastError()) );
        return errBuf;
    }
    return NULL;
}

int dataValidate( char * lck, char * shared, char * local, char * run )
{
    if ( ! validateFolder( lck ) )      return 1;
    if ( ! validateFolder( shared ) )   return 2;
    if ( ! validateFolder( local ) )    return 3;
    if ( ! validateCommand( run ) )     return 4;

    return 0;
}

BOOL readData( void )
        // returns TRUE if data loaded from INI file
        //        FALSE - indicates a configuration needs to be done!
{
    if ( ! validateCommand( getIniName() )  )        return FALSE;

    char buf[ 1024 ];

    GetPrivateProfileString( SECTION, REMOTE_LOCK_KEY,         "", buf, sizeof( buf ), getIniName() );
    setRemoteLock( buf );

    GetPrivateProfileString( SECTION, REMOTE_SHARED_DATA_KEY,  "", buf, sizeof( buf ), getIniName() );
    setRemoteSharedData( buf );

    GetPrivateProfileString( SECTION, LOCAL_DATA_KEY,          "", buf, sizeof( buf ), getIniName() );
    setLocalData( buf );

    GetPrivateProfileString( SECTION, RUN_COMMAND_KEY,         "", buf, sizeof( buf ), getIniName() );
    setRunCommand( buf );

    GetPrivateProfileString( SECTION, RUN_COMMAND_OPTIONS_KEY, "", buf, sizeof( buf ), getIniName() );
    setRunCommandOption( buf );

    return TRUE;
}
