/////////////////////////////////////////////////////////////////////////////
//                                                            (c) 1998/9 SNS
//  WinAppControl
//                  Supporting code for TreeShare/PilotShare
//
//      Authors: George A. Smith & John R. Coleman
//      Why:     We felt the need.
//
//  LICENCE CONDITIONS:
//
//      The Authors consent to GNU General Public Licence for this code.
//      Other Licencing arrangements are possible.
//
//  NOTES:
//
//      Please notify us of any problems you find in the code.
//      Please notify us of any changes you make to this code that you
//              subsequently redistribute.
//      If you use this program, please tell us...
//
//      We can be contacted at:     PilotShare@SNS.to
//                          or:     TreeShare@SNS.to
//
//      Info:       http://SNS.to/PilotShare
//
//      If you redistribute, please leave the above in!
//
/////////////////////////////////////////////////////////////////////////////

#include "IncCmn.h"
//---------------------------------------------------------------------------
#include "WinAppControl.h"
//---------------------------------------------------------------------------

#include "WinTools.h"
#include "WinErr.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

WinAppControl::WinAppControl( char * appPath, char * cmdLine )
{
    char drive[MAXDRIVE];
    char dir[MAXDIR];

    fnsplit( appPath, drive, dir, NULL, NULL);

    appDir        = newSpace( dir );
    appDrive      = newSpace( drive );
    appExecutable = newSpace( appPath );
    appCmdLine    = newSpace( cmdLine );

    *drive = (char)toupper( *drive );
    if ( isalpha( *drive ) )  dosDrive = *drive - 'A';
    else                      dosDrive = -1;

    errMsg = NULL;
    AppLaunched = false;
}

WinAppControl::~WinAppControl(void)
{
    errMsg          = dynFree( errMsg );
    appDir          = dynFree( appDir );
    appDrive        = dynFree( appDrive );
    appExecutable   = dynFree( appExecutable );
    appCmdLine      = dynFree( appCmdLine );
}

void WinAppControl::setErr( char * msg1, char * msg2 )
{
    errMsg = dynSpace( errMsg, msg1, msg2 );
}

void WinAppControl::setErr( char * msg )
{
    setErr( msg, "" );
}

char * WinAppControl::getErr( void )
{
    return errMsg;
}

bool WinAppControl::changeToAppDir( void )
{
    if ( -1 != dosDrive )
    {
        setdisk( dosDrive );
        if (getdisk() != dosDrive)  return false;
    }
    if ( 0 == chdir( appDir )  )    return true;

    return false;
}

bool WinAppControl::launch_normal( void )
        //  true indicates success
        // false indicates error, see getErr
{
    changeToAppDir();

    si.cb           = sizeof( si ) ;
    si.lpReserved   = NULL ;
    si.lpDesktop    = NULL ;
    si.lpTitle      = NULL ;
    si.dwFlags      = STARTF_USESHOWWINDOW ; // | STARTF_USESTDHANDLES ;
    si.wShowWindow  = SW_NORMAL ;
    si.cbReserved2  = 0 ;
    si.lpReserved2  = NULL ;
    // si.hStdInput    = ...
    // si.hStdOutput   = ...
    // si.hStdError    = ...

    char * zCmd = newSpace( appExecutable, " ", appCmdLine );

    AppLaunched = CreateProcess( NULL,                                     // Module Name
                                zCmd ,                                // Command Line
                                NULL ,                                      // Process Security
                                NULL ,                                      // Thread Security
                                FALSE ,                                     // Inherit our handles
                                DETACHED_PROCESS | NORMAL_PRIORITY_CLASS ,  // Creation Flags
                                NULL ,                                      // New Environment Block
                                NULL ,                                      // Current Dir Name
                                &si ,                                       // STARTUPINFO (see above)
                                &proc_info ) ;                               // Result in our PROCESS_INFORMATION
    delete [] zCmd;

    if ( ! AppLaunched )
        setErr( "Unable to run command : ",  DecodeWindowsError( GetLastError() ) );

    return AppLaunched;
}

bool WinAppControl::isRunning( void )
{
    if ( AppLaunched )
    {
        DWORD status;
        GetExitCodeProcess( proc_info.hProcess, &status );
        AppLaunched = STILL_ACTIVE == status;
    }
    return AppLaunched;
}
