/////////////////////////////////////////////////////////////////////////////
//                                                            (c) 1998/9 SNS
//  WinTools
//                  Supporting code for TreeShare/PilotShare
//
//      Authors: George A. Smith & John R. Coleman
//      Why:     We felt the need.
//
//  LICENCE CONDITIONS:
//
//      The Authors consent to GNU General Public Licence for this code.
//      Other Licencing arrangements are possible.
//
//  NOTES:
//
//      Please notify us of any problems you find in the code.
//      Please notify us of any changes you make to this code that you
//              subsequently redistribute.
//      If you use this program, please tell us...
//
//      We can be contacted at:     PilotShare@SNS.to
//                          or:     TreeShare@SNS.to
//
//      Info:       http://SNS.to/PilotShare
//
//      If you redistribute, please leave the above in!
//
/////////////////////////////////////////////////////////////////////////////

#include "IncCmn.h"
//---------------------------------------------------------------------------
#include "WinTools.h"
//---------------------------------------------------------------------------

#include <shlobj.h>
#include <io.h>
#include <dos.h>
#include <stdio.h>
#include <ctype.h>

#include "WinErr.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

char * newSpace( char * data, char * data2, char *data3, char *data4 )
{
    if ( NULL == data  )    data  = "";
    if ( NULL == data2 )    data2 = "";
    if ( NULL == data3 )    data3 = "";
    if ( NULL == data4 )    data4 = "";

    int len = strlen(data) + strlen(data2) + strlen(data3) + strlen(data4);
    char * newSpace = new char[ len + 2 ];
    strcpy( newSpace, data  );
    strcat( newSpace, data2 );
    strcat( newSpace, data3 );
    strcat( newSpace, data4 );

    *(newSpace + len + 1) = 0;

    return newSpace;
}

char * newSpace( char * data, char * data2, char * data3)
{
    return newSpace( data, data2, data3, "" );
}

char * newSpace( char * data, char * data2)
{
    return newSpace( data, data2, "", "" );
}

char * newSpace( char * data )
{
    return newSpace( data, "", "", "" );
}

char * dynFree( char * ptr )
{
    if ( NULL != ptr )  delete [] ptr;
    return NULL;
}

char * dynSpace( char * old, char * data, char * data2, char *data3, char *data4 )
{
    char * rv = newSpace( data, data2, data3, data4 );
    dynFree( old );
    return rv;
}

char * dynSpace( char * old, char * data, char * data2, char * data3)
{
    return dynSpace( old, data, data2, data3, "" );
}

char * dynSpace( char * old, char * data, char * data2)
{
    return dynSpace( old, data, data2, "", "" );
}

char * dynSpace( char * old, char * data )
{
    return dynSpace( old, data, "", "", "" );
}

//---------------------------------------------------------------------------

bool errorDialog( HWND hWnd, char * title, char * message )
    // returns false
{
    int flags = MB_ICONSTOP;
    if ( NULL != hWnd )     flags |= MB_TASKMODAL;

    MessageBox( hWnd, message, title, flags );
    return false;
}

BOOL fileExists( char * fileName )      {   return ( access( fileName , 0)? FALSE : TRUE );   }
BOOL fileReadable( char * fileName )    {   return ( access( fileName , 4)? FALSE : TRUE );   }
BOOL fileWritable( char * fileName )    {   return ( access( fileName , 2)? FALSE : TRUE );   }

// all return TRUE if file found/readable
BOOL validateFile( char * fileName )    {   return fileExists( fileName );  }
BOOL validateCommand( char * exeCmd )   {   return fileReadable( exeCmd );  }
BOOL validateFolder( char * folder )    {   return fileExists( folder );    }

bool deleteFile( char * filename )
    // return false if error
    //        true if file deleted
{
    _rtl_chmod( filename, 1, FA_ARCH );
    remove( filename );

    if ( !fileExists( filename ) )      return true;

    return false;
}

//---------------------------------------------------------------------------

bool delTree(char * src)
    // true is success
    // false is error
{
    if ( ! validateFolder( src ) )  return FALSE;

    SHFILEOPSTRUCT  shfos;

    shfos.hwnd = NULL;
    shfos.wFunc = FO_DELETE;
    shfos.pFrom = src;
    shfos.pTo = "";
    shfos.fFlags = /* FOF_ALLOWUNDO|*/ FOF_FILESONLY|FOF_NOCONFIRMATION;
    shfos.lpszProgressTitle = "Deleting old tree";
    shfos.fAnyOperationsAborted = FALSE;

    if ( SHFileOperation( &shfos ) )
        return errorDialog( NULL, "Deleting", DecodeWindowsError( GetLastError() ) );
    if ( shfos.fAnyOperationsAborted )
        return errorDialog( NULL, "Deleting", "User aborted delete" );

    return true;
}

bool copyTree(char * src, char * dst)
    // true is success
    // false is error
{
    char * from = newSpace( src, "\\*.*" );

    SHFILEOPSTRUCT  shfos;

    shfos.hwnd = NULL;
    shfos.wFunc = FO_COPY;
    shfos.pFrom = from;
    shfos.pTo = dst;
    shfos.fFlags = FOF_NOCONFIRMATION|FOF_NOCONFIRMMKDIR;
    shfos.lpszProgressTitle = "Copying share data";
    shfos.fAnyOperationsAborted = FALSE ;

    bool ret = true;

    if ( SHFileOperation( &shfos ) )
        ret = errorDialog( NULL, "Copying",DecodeWindowsError( GetLastError() ) );
    else
        if ( shfos.fAnyOperationsAborted )
            ret = errorDialog( NULL, "Copying", "User aborted copy" );

    delete [] from;
    return ret;
}

bool renTree(char * src, char * dst)
    // true is success
    // false is error
{
    SHFILEOPSTRUCT  shfos;

    shfos.hwnd = NULL;
    shfos.wFunc = FO_RENAME;
    shfos.pFrom = src;
    shfos.pTo = dst;
    shfos.fFlags = FOF_NOCONFIRMATION|FOF_NOCONFIRMMKDIR;
    shfos.lpszProgressTitle = "Copying share data";
    shfos.fAnyOperationsAborted = FALSE ;

    if ( SHFileOperation( &shfos ) )
        return errorDialog( NULL, "renaming", DecodeWindowsError( GetLastError() ) );
    if ( shfos.fAnyOperationsAborted )
        return errorDialog( NULL, "renaming", "User aborted copy" );

    return true;
}

bool folderCopyWithOldSave( char * src, char * dest )
    // true is success
    // false is error
{
    char * srcDir  = newSpace( src );
    char * regDest = newSpace( dest );
    char * savDest = newSpace( dest, ".saved" );
    char * delDest = newSpace( dest, ".2delete" );

    if ( validateFolder( delDest ) )
        if ( !delTree( delDest ) )
            return errorDialog( NULL, "Deleting old save folder", "Error occured during delete" );

    if ( validateFolder( savDest ) )
        if ( !renTree( savDest, delDest ) )
            return errorDialog( NULL, "Renaming save to old save", "Error occuried during rename" );

    if ( !renTree( regDest, savDest ) )
        return errorDialog( NULL, "Renaming folder to save", "Error occuried during rename" );

    if ( !copyTree( srcDir, regDest ) )
        return errorDialog( NULL, "Copying", "Error occuried during filecopy" );

    if ( validateFolder( delDest ) )
        if ( !delTree( delDest ) )
            return errorDialog( NULL, "Deleting old save folder", "Error occured during delete" );

    delete delDest;
    delete savDest;
    delete regDest;
    delete srcDir;
    return true;
}

//---------------------------------------------------------------------------

char * browseForFolder( char * deflt, char * title, HWND parent )
{
    BROWSEINFO fbs;
    static char selectedPath[MAX_PATH+2];

    *selectedPath = 0;

    fbs.hwndOwner = parent;
    fbs.pidlRoot = NULL;
    fbs.pszDisplayName =selectedPath;
    fbs.lpszTitle = title;
    fbs.ulFlags = BIF_RETURNONLYFSDIRS;
    fbs.lpfn = NULL;
    fbs.lParam = NULL;
    fbs.iImage = 0;

    LPITEMIDLIST pidlBrowse = SHBrowseForFolder( &fbs );

    if ( NULL != pidlBrowse )
        if (SHGetPathFromIDList(pidlBrowse, selectedPath))
            return selectedPath;

    return deflt;
}

char * getBaseName( char *full)
    // return dynamic space that contain base filename from "full"
    //
{
    for ( char * bp = full; *bp; bp++ )
        if ( ('\\' == *bp)||('/' == *bp ) )
            full = bp + 1;

    char * rv = newSpace( full, "." );

    *strchr( rv, '.' ) = 0;

    if ( *rv )
    {
        for ( char * tp = rv + 1; *tp; tp++ )
            *tp = (char)tolower( *tp );

        *rv = (char)toupper( *rv );
    }

    return rv;
}

char * getPath( char *full )
    // no trailing slash
    // can be zero length
{
    char * rv = newSpace( full );
    char * tp = rv;

    for ( char * bp = tp; *bp; bp++ )
        if ( ('\\' == *bp)||('/' == *bp ) )
            tp = bp;

    *tp = 0;

    return rv;
}

char * getPathAndBaseName( char *full )
    // return dynamic space that contains path and base filename from "full"
    //    extension has been removed
    // can return 0 length string
{
    char * path = getPath( full );
    char * base = getBaseName( full );

    char * rv = newSpace( path, "\\", base );

    dynFree( path );
    dynFree( base );

    return rv;
}

struct ffblk * findFile( char * fp )
{
    static struct ffblk ffblk;

    if ( findfirst( fp, &ffblk, 0 ) )   return NULL;

    return &ffblk;
}
