/////////////////////////////////////////////////////////////////////////////
//                                                            (c) 1998/9 SNS
//  cLogo
//                  Supporting code for TreeShare/PilotShare
//
//      Authors: George A. Smith & John R. Coleman
//      Why:     We felt the need.
//
//  LICENCE CONDITIONS:
//
//      The Authors consent to GNU General Public Licence for this code.
//      Other Licencing arrangements are possible.
//
//  NOTES:
//
//      Please notify us of any problems you find in the code.
//      Please notify us of any changes you make to this code that you
//              subsequently redistribute.
//      If you use this program, please tell us...
//
//      We can be contacted at:     PilotShare@SNS.to
//                          or:     TreeShare@SNS.to
//
//      Info:       http://SNS.to/PilotShare
//
//      If you redistribute, please leave the above in!
//
/////////////////////////////////////////////////////////////////////////////

#include "IncCmn.h"
//---------------------------------------------------------------------------
#include "cLogo.h"
//---------------------------------------------------------------------------

#include "cRegInfo.h"
#include "cStartLock.h"
#include "cFinishLockGone.h"
#include "cFinishLockStolen.h"

#include "Configure.h"
#include "WinTools.h"
#include "TokenLock.h"
#include "PersistentData.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)
#pragma resource "*.dfm"
TLogo *Logo;

//---------------------------------------------------------------------------

__fastcall TLogo::TLogo(TComponent* Owner) : TForm(Owner)
{
    curState = ST_VIRGIN;

    userName =
    errTitle =
    errMsg   =
    ourName  = NULL;
    zApp     = NULL;
}

//---------------------------------------------------------------------------

void __fastcall TLogo::lb_TitleClick(TObject *Sender)
{
    TRegInfo *reg = new TRegInfo( this, ourBaseName );
    reg->ShowModal();
    delete( reg );
}

//---------------------------------------------------------------------------

void __fastcall TLogo::FormShow(TObject *Sender)
{
    zTimer->Enabled = true;
}

//---------------------------------------------------------------------------

void __fastcall TLogo::clearFlags( void )
{
    modeCancel = modeReadOnly = modeDeleteLock = false;
}

//---------------------------------------------------------------------------

void __fastcall TLogo::setErr( char * title, char * msg1, char * msg2 )
{
    errTitle = dynSpace( errTitle, title );
    errMsg   = dynSpace( errMsg,   msg1, msg2 );
    curState = ST_ERROR;
}

//---------------------------------------------------------------------------

void __fastcall TLogo::setErr( char * title, char * msg )
{
    setErr( title, msg, "" );
}

//---------------------------------------------------------------------------

char * __fastcall TLogo::getOurName( char *full )
{
    char * rv = getBaseName( full );
    if ( 0 == strlen( rv ) )
        rv = dynSpace( rv, "TreeShare" );

    return rv;
}

char * __fastcall TLogo::getOurPathAndName( char *full )
{
    char * rv = getPathAndBaseName( full );
    if ( 0 == strlen( rv ) )
        rv = dynSpace( rv, "TreeShare" );

    return rv;
}

//---------------------------------------------------------------------------

void __fastcall TLogo::editIni( void )
{
    clearFlags();
    TTreeShare *cfg = new TTreeShare( this );
    cfg->ShowModal();
    delete cfg;
}

//---------------------------------------------------------------------------

bool __fastcall TLogo::obtainLock( void )
        // true - user allows to proceed
        // false - failed
{
    lockPath = getRemoteLock();

    while( ! getLock( lockPath, userName ) )
    {
        TStartLock * st = new TStartLock( this );
        st->setFields( isExistingLock( lockPath ), lockTime(lockPath, isExistingLock( lockPath )) );
        st->ShowModal();
        delete st;
        if ( modeCancel )       return false;

        if ( modeReadOnly )     break;
        if ( modeDeleteLock )
        {
            deleteLock( lockPath, isExistingLock(lockPath) );
            modeDeleteLock = false;
        }
    }

    return true;
}

bool __fastcall TLogo::verifyStillLocked( void )
        // true -- our lock is still valid
        // false -- lock not valid/aborted
{
    if ( ! validateLock( lockPath, userName ) )
    {
        char * lockName = isExistingLock( lockPath );
        if ( NULL != lockName)
        {
            TFinishLockStolen *fls = new TFinishLockStolen(this);
            fls->setFields( lockName, lockTime(lockPath, lockName) );
            fls->ShowModal();
            delete fls;
            if ( modeDeleteLock )
            {
                if ( !deleteLock( lockPath, lockName ) )
                {
                    setErr( "Lock Delete", "Can't delete existing lock : ", lockName );
                    return false;
                }
                modeDeleteLock = false;
            }
        }
        else
        {
            TFinishLockGone *fls = new TFinishLockGone(this);
            fls->ShowModal();
            delete fls;
        }
        if ( modeCancel )   return false;
    }

    return true;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

void __fastcall TLogo::doVirgin( void )
        //  Setup USerName
        //        Inifile
        //        Our Program name
{
    userName = newSpace( ParamStr(1).c_str() );

    char * tp = newSpace( ParamStr(0).c_str() );

    ourBaseName = getOurPathAndName( tp );
    buildIniName( tp );

    lb_Title->SetTextBuf( ourName = getOurName( tp ) );
    Caption  = ourName;

    clearFlags();
    curState = ST_VALIDATE_INI;
}

//---------------------------------------------------------------------------

void __fastcall TLogo::doValidateIni( void )
{
    bool iniRead = readData();

    if (  (0 == strlen( userName ))     // no User ID
        ||(!iniRead) )                  // failed INIread
    {                                           // just Edit INI and exit
        editIni();
        curState = ST_DONE;
        return;
    }

    if ( dataValidate( getRemoteLock(), getRemoteSharedData(), getLocalData(), getRunCommand()) )
        editIni();

    curState = modeCancel ? ST_DONE : ST_GET_LOCK;
}

//---------------------------------------------------------------------------

void __fastcall TLogo::doGetLock( void )
{
    curState = obtainLock() ? ST_GET_DATA : ST_DONE;
}

//---------------------------------------------------------------------------

void __fastcall TLogo::doGetData( void )
{
    curState = ST_LAUNCH_APP;
    if ( ! folderCopyWithOldSave( getRemoteSharedData(), getLocalData() ) )
        setErr( "Copy Remote Data", "Error copying remote shared data" );
}

//---------------------------------------------------------------------------

void __fastcall TLogo::doLaunchApp( void )
{
    zApp = new WinAppControl( getRunCommand(), getRunCommandOption() );
    curState = ST_APP_WAIT;
    if ( ! zApp->launch_normal() )
        setErr( "Lanuching Application", zApp->getErr() );
}

//---------------------------------------------------------------------------

void __fastcall TLogo::doAppWait( void )
{
    if ( NULL == zApp )
        setErr( "Internal Error", "Lost application handle" );
    else
        if ( ! zApp->isRunning() )
            curState = ST_VALIDATE_LOCK;
}

//---------------------------------------------------------------------------

void __fastcall TLogo::doValidateLock( void )
{
    curState = ST_DONE;
    if ( ! modeReadOnly )
        if ( verifyStillLocked() )
            curState = ST_RETURN_DATA;
}

//---------------------------------------------------------------------------

void __fastcall TLogo::doReturnData( void )
{
    curState = ST_DONE;
    if ( ! folderCopyWithOldSave( getLocalData(), getRemoteSharedData() ) )
        setErr( "Copy Local Data", "Error copying local data back to Shared Data Folder" );
}

//---------------------------------------------------------------------------

void __fastcall TLogo::doDone( void )
{
    zTimer->Enabled = false;
    if ( strlen( userName ) )   deleteLock( lockPath, userName );
    Close();
}

//---------------------------------------------------------------------------

void __fastcall TLogo::doError( void )
{
    errorDialog( Handle, errTitle, errMsg );
    curState = ST_DONE;
}

//--------------------------------------------------------------------------

void __fastcall TLogo::setCancel( void )
{
    modeCancel = true;
}

//---------------------------------------------------------------------------

void __fastcall TLogo::setDeleteLock( void )
{
    modeDeleteLock = true;
}

//---------------------------------------------------------------------------

void __fastcall TLogo::setReadOnly( void )
{
    modeReadOnly = true;
}

void __fastcall TLogo::FormDestroy(TObject *Sender)
{
    userName    = dynFree( userName );
    errTitle    = dynFree( errTitle );
    errMsg      = dynFree( errMsg );
    ourName     = dynFree( ourName );
    ourBaseName = dynFree( ourBaseName );
    if ( NULL != zApp )     delete zApp;
}

//---------------------------------------------------------------------------

void __fastcall TLogo::zTimerTimer(TObject *Sender)
{
    zTimer->Enabled = false;
    switch( curState )
    {
        default:
        case ST_VIRGIN:         doVirgin();             break;
        case ST_VALIDATE_INI:   doValidateIni();        break;
        case ST_GET_LOCK:       doGetLock();            break;
        case ST_GET_DATA:       doGetData();            break;
        case ST_LAUNCH_APP:     doLaunchApp();          break;
        case ST_APP_WAIT:       doAppWait();            break;
        case ST_VALIDATE_LOCK:  doValidateLock();       break;
        case ST_RETURN_DATA:    doReturnData();         break;
        case ST_DONE:           doDone();               return;

        case ST_ERROR:          doError();              break;
    }
    zTimer->Enabled = true;
}

//---------------------------------------------------------------------------
